// Copyright (C) Microsoft. All rights reserved.
precision highp float;
uniform lowp sampler2D s_customDistance;
uniform vec4 FillColor;
uniform vec4 StrokeColor;
uniform float CircleRadius;
varying vec3 vPosCopy;
varying float vDistance;
varying float vRingWidthRatio;
varying vec2 vOffset;
#define RING_INTERSECT_ALPHA 0.7
#define RING_DEPTH_FADE_LIMIT 0.5
#define kLabelDistanceToZero 1.0/6.0
#define kPointsAndLabelsDistanceFadeCutoff 0.0
float PositionToDistanceFade(vec3 pos,float localZ,float distanceToZero)
{
vec2 pc=pos.xy/pos.z;
vec2 texcoords=vec2((pc.x+1.0)*0.5,
(pc.y+1.0)*0.5);
vec4 reconstruct=vec4(
65536.0*255.0/16777216.0,
256.0*255.0/16777216.0,
1.0*255.0/16777216.0,
0.0*255.0/16777216.0);
lowp vec4 value=texture2D(s_customDistance,texcoords);
float val=dot(value,reconstruct);
return clamp(1.0-((localZ-val)/(val*distanceToZero)),0.0,1.0);
}
float PointsAndLabelsDistanceFade(vec3 pos,float localZ,float distanceToZero,float disableDepthFade)
{
float depthFadeResult=PositionToDistanceFade(pos,localZ,distanceToZero);
return max(disableDepthFade,depthFadeResult<kPointsAndLabelsDistanceFadeCutoff?0.0:depthFadeResult);
}
void main()
{
float depthFade=PointsAndLabelsDistanceFade(
vPosCopy,
vDistance,
kLabelDistanceToZero,
0.0);
vec4 color;
float len=length(vOffset);
if(len<=1.0-1.5*vRingWidthRatio)
{
color=vec4(FillColor.xyz,FillColor.w*depthFade);
}
else if(len<=1.0-vRingWidthRatio)
{
float t=(len-(1.0-1.5*vRingWidthRatio))/(.5*vRingWidthRatio)/2.0+.5;
float aaFade=8.0*pow(1.0-t,3.0);
color=vec4(FillColor.xyz,FillColor.w*depthFade*aaFade);
}
else if(len<=1.0)
{
if(depthFade<RING_DEPTH_FADE_LIMIT)
{
color=vec4(0.462,0.643,.702,RING_INTERSECT_ALPHA);
}
else
{
color=vec4(StrokeColor.xyz,StrokeColor.w*depthFade);
}
float t=(len-(1.0-vRingWidthRatio))/vRingWidthRatio;
float aaFade=8.0*pow(t<.5?t:1.0-t,3.0);
color.a*=aaFade;
}
gl_FragColor=color;
}
