package com.microsoft.jars.VoiceSearch;
import java.util.Vector;

/**
 * The following class represents a SNS phrase.
 */
public class SnsPhrase {

    /** Elements of this SNS phrases. */    
    private Vector<SnsPhraseElement> elements;
    
    // search result
    public static final int SEARCH_RESULT_NOT_YET_DOWNLOADED = 0;
    public static final int SEARCH_RESULT_DOWNLOADED = 1;
    public static final int SEARCH_RESULT_PARSED = 2;
    private int searchResultState = SEARCH_RESULT_NOT_YET_DOWNLOADED;

    /**
     * Allocates a new SNS phrase.
     */
    public SnsPhrase() {
        elements = new Vector<SnsPhraseElement>();
    }

    private String text;
    
    /**
     * Adds an element to this SNS phrase.
     * 
     * @param element
     *                the element to add.
     */
    public void addElement(SnsPhraseElement element) {
        elements.addElement(element);
    }

    /**
     * Gets the element at the given index.
     * 
     * @param index
     *                the index of the element to get.
     * @return the element at the given index.
     */
    public SnsPhraseElement getElement(int index) {
        return (SnsPhraseElement) elements.elementAt(index);
    }

    /**
     * Gets the number of elements in this SNS phrase.
     * 
     * @return the number of elements in this SNS phrase.
     */
    public int getElementCount() {
        return elements.size();
    }

    /**
     * Gets the <code>String</code> which represents this SNS phrase.
     * 
     * @return the <code>String</code> which represents this SNS phrase.
     */
    public String getText() {
        if (text == null)
        {
            StringBuffer stringBuffer = new StringBuffer();
            int size = elements.size();
            boolean mustAddTrailingSpace = false;
            for (int i = 0; i < size; i++) {
                SnsPhraseElement element = (SnsPhraseElement) elements.elementAt(i);
                if (mustAddTrailingSpace) {
                    stringBuffer.append(' ');
                }
                mustAddTrailingSpace = element.mustAddTrailingSpace();
                if (element.mustConsumeLeadingSpace()
                        && stringBuffer.charAt(stringBuffer.length() - 1) == ' ') {
                    stringBuffer.deleteCharAt(stringBuffer.length() - 1);
                }
                stringBuffer.append(element.getText());
            }
            text = stringBuffer.toString();
        }
        return text;
    }
    
    /**
     * Get the state of the search result for this phrase.
     */
    public int getSearchResultState()
    {
        synchronized(this)
        {
            return searchResultState;
        }
    }   
}
