/**
 * Copyright (c) Microsoft Corporation 2006-2008. All Rights Reserved.
 * Information Contained Herein is Proprietary and Confidential.
 */

package com.microsoft.jars.VoiceSearch;
import java.util.Vector;

/**
 * The following class represents a SNS result.
 */
public class SnsResult {

    /** Name of this SNS result. */
    private String name;

    /** Phrases for this SNS result. */
    private Vector<SnsPhrase> phrases;

    /**
     * Allocates a new <code>SnsResult</code>.
     * 
     * @param name
     *                the name of this SNS result.
     */
    public SnsResult(String name) {
        this.name = name;
        phrases = new Vector<SnsPhrase>();
    }

    /**
     * Gets the name of this SNS result.
     * 
     * @return the name of this SNS result.
     */
    public String getName() {
        return name;
    }

    /**
     * Adds a phrase to this SNS result.
     * 
     * @param phrase
     *                the phrase to add.
     */
    public void addPhrase(SnsPhrase phrase) {
        phrases.addElement(phrase);
    }

    /**
     * Gets the phrase at the given index.
     * 
     * @param index
     *                the index of the phrase to get.
     * @return the phrase at the given index.
     */
    public SnsPhrase getPhrase(int index) {
        return (SnsPhrase) phrases.elementAt(index);
    }

    /**
     * Gets the number of phrases in this SNS result.
     * 
     * @return the number of phrases in this SNS result.
     */
    public int getPhraseCount() {
        return phrases.size();
    }

    /**
     * Get the index of a phrase having the supplied text.
     * @param text <description>
     * @return <description>
     */
    public int getPhraseIndex(String text)
    {
        for (int i=phrases.size()-1; i>=0; i--)
        {
            SnsPhrase phrase = (SnsPhrase)phrases.elementAt(i);
            if (phrase.getText().equals(text))
            {
                return i;
            }
        }
        return -1;
    }
}
