//
/*
 * VoiceSearchRequest.java
 *
 * ?<your company here>, 2003-2008
 * Confidential and proprietary.
 */
package com.microsoft.jars.VoiceSearch;

import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Hashtable;
import java.util.Vector;
import com.microsoft.mobileexperiences.bing.httpthreadpool.HttpRequest;
import com.microsoft.mobileexperiences.bing.httpthreadpool.HttpResponseBase;
import com.microsoft.mobileexperiences.bing.httpthreadpool.IHttpResult;

import android.content.Context;
import android.content.res.AssetManager;
import android.location.Location;
import android.util.Log;

/**
 * The <code>VoiceSearchRequest</code> class extends ServerRequestBase class and
 * represents a voice search query.
 */
public class VoiceSearchRequest extends HttpRequest {

	private final static int REQUEST_FORMAT_JSON = 3; // use JSON

	private String[] providers;
	private String mAndroidVersion;
	private int audioFormat;
	//private String LogTag = "VoiceSearch";
	private String mLastGUID;
	private String mLastResult;
	private String mCulture = "en-US";
	private String mPlatform;
	private String mPartener;
	private String mSearchTemplate;
	private boolean requireResults = true;
	
	/**
	 * Constructor.
	 */
	public  VoiceSearchRequest(
			String[] providers, 
			String url,
			String androidVersion,
			String culture,
			String platform,
			String partner,
			String searchTemplate,
			int audioFormat,
			boolean requireResults) {
		
		super(url, HttpRequest.HttpMethod.HTTP_METHOD_POST);
		this.setChunkedStreamingMode(true);
		
		this.mPartener = partner;
		this.mPlatform = platform;
		this.mCulture = culture;
		this.mLastGUID = java.util.UUID.randomUUID().toString();		
		this.providers = providers;
		this.mAndroidVersion = androidVersion;
		this.audioFormat = audioFormat;
		this.mSearchTemplate = searchTemplate;
		this.requireResults = requireResults;
		
	}	
	
	private void writeExtraBytes(DataOutputStream dos, String extraBytes)
			throws IOException {
		byte[] data = extraBytes.getBytes();
		dos.writeShort(revertShort(data.length));
		dos.write(data, 0, data.length);		
	}
	
	/**
	 * Used to set HTTP request content.
	 * 
	 * @param out
	 *            The DataOutputStream of the connection.
	 * @throws CustomException
	 */
	@Override
	public void streamContentOut(OutputStream outputStream)
	{
		DataOutputStream out = new DataOutputStream(new BufferedOutputStream(outputStream));
		try {
			writeProviders(out, providers);
			writeOptions(out);
			writeRequestFormat(out, REQUEST_FORMAT_JSON);
			writeExtraBytes(out, this.mSearchTemplate);
			
			out.writeShort(revertShort(this.audioFormat));				
			writeVoiceStream(out);			
			
			out.flush();
		} catch (IOException e) {
			
		}
	}
	
	public void writeVoiceStream(DataOutputStream outputStream) {}

	/**
	 * Write the request format. We're using SOAP.
	 * 
	 * @param dos
	 *            <description>
	 * @param requestFormat
	 *            <description>
	 */
	private void writeRequestFormat(DataOutputStream dos, int requestFormat)
			throws IOException {
		dos.writeShort(revertShort(requestFormat));
	}	
		
	/**
	 * Writes a string followed by a CR-LF separator to an output stream.
	 * 
	 * @param dos
	 *            the output stream
	 * @param str
	 *            the string to write
	 * @throws IOException
	 */
	private void writeProviders(DataOutputStream dos, String[] providers)
			throws IOException {
		writeStrings(dos, providers);
	}

	private void writeOptions(DataOutputStream dos) throws IOException {
		writeStrings(dos, makeOptions());;
		this.mLastGUID = "";
		this.mLastResult = "";
	}

	private String[] makeOptions() {
		Vector<String> options = new Vector<String>();
		options.addElement("Culture=" + this.mCulture);
		options.addElement("ClientVersion="	+ this.mAndroidVersion);
		options.addElement("ClientPlatform=" + this.mPlatform);
		options.addElement("PartnerId=" + this.mPartener);
		options.addElement("ReturnPerformanceData=true");
		if (requireResults)
			options.addElement("NumResults=1");
		else
			options.addElement("NumResults=0");
        options.addElement("NumAlternates=0");
		if (this.mLastGUID != null && mLastGUID.length() != 0
				&& this.mLastResult != null && this.mLastResult.length() != 0) {
			options.addElement("LastRecoGuid=" + this.mLastGUID);
			options.addElement("LastResult=" + this.mLastResult);
		}

		String[] list = new String[options.size()];
		options.copyInto(list);
		return list;
	}

	private void writeStrings(DataOutputStream dos, String[] strings)
			throws IOException {
		for (int i = 0; i < strings.length; i++) {
			writeCrLfString(dos, strings[i]);
		}
		writeCrLfString(dos, null); // end of string list
	}

	/**
	 * Writes a string followed by a CR-LF separator to an output stream.
	 * 
	 * @param dos
	 *            the output stream
	 * @param str
	 *            the string to write
	 * @throws IOException
	 */
	private void writeCrLfString(DataOutputStream dos, String str)
			throws IOException {

		if (str != null && str.length() > 0) {
			dos.write(str.getBytes());
		}
		dos.write(13);
		dos.write(10);
	}

	/**
	 * Reverts a short by swapping most and least significant byte, for
	 * little-endian protocols. We store a short in an int because there is no
	 * unsigned short in java.
	 * 
	 * @param s
	 *            the short value to revert
	 * @return the reverted short value
	 */
	public static int revertShort(int s) {
		s = s & 0xffff; // make sure it's a short
		int l = s & 0xFF;
		int h = s >>> 8;
		return ((l << 8) | h);
	}

	/**
	 * Get the string representation of the request.
	 * 
	 * @return <code>String</code> request String.
	 */
	public String toString() {
		return "";
	}


	//default decode function. Could be overrided.
	protected HttpResponseBase decode(InputStream stream, Hashtable<String, String> headers)
	{
		SnsResponse snsResponse = new SnsResponse();
		//Log.i(LogTag, "Decode response");
		if(stream != null )
		{
			try {			
				snsResponse.init(stream);
			
			} catch (Exception e) {
				
			}
		}
		else
		{
			//Log.i(LogTag, "Decode response null");
		}
		return snsResponse;
	}
	


	@Override
	public void setHttpHeaders() {
		this.addHttpHeader("Accept-Encoding", "gzip");		
	}
}
