// TODO: This class is in the HttpThreadPool.jar temporarily. It should be replace by
// the cache supplied by the shell eventually.

package com.microsoft.mobileexperiences.bing.httpthreadpool;


import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;
import java.util.UUID;

import com.microsoft.mobileexperiences.bing.httpthreadpool.HttpDefaultResponse;

import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteStatement;
import android.provider.BaseColumns;
import android.util.Log;

public class FileDBCacheHelper extends SQLiteOpenHelper {
    private static final String LOGTAG = "FileDBCache";
    private static final boolean debugMode = false;
    private static String DB_NAME = "nativecache.db";

    private static String DB_PATH = "/data/data/%s/databases/" + DB_NAME;
    private static String FILE_CACHE_PAHT = "/data/data//%s/FileCache";

    private SQLiteStatement updateFileQuery;
    private SQLiteStatement insertFileQuery;

    // Table name
    private static String TABLE_NAME = "cache";

    // Columns
    private static final String URLKEY = "url";

    private static final String URLKEYINDEX = "urlindex";

    private static final String FILENAME = "filename";

    private static final String TIMESTAMP = "timestamp";

    private static final String EXPIRATION = "expiration";

    private static final int MAX_SIZE = 1000;
    private static final int DEL_NUM = 80;

    private int totalCachedFiles;

    private String mDBPath;
    private String filePath;

    private SQLiteDatabase cacheDB;

    private static FileDBCacheHelper instance;

    private static Context context;

    public static void setContext(Context context)
    {
        FileDBCacheHelper.context = context;
    }

    private FileDBCacheHelper(Context context) {
        super(context, DB_NAME, null, 1);    	

        boolean pathExist = true;

        filePath = String.format(FILE_CACHE_PAHT, context.getPackageName());

        File tileCachePath = new File(filePath); 
        if (!tileCachePath.exists())
        {
            pathExist = tileCachePath.mkdir();
        }

        if (pathExist)
        {
            mDBPath = String.format(DB_PATH, context.getPackageName());
            File file = new File(mDBPath);

            try
            {
                if (!file.exists()) 
                {
                    cacheDB = this.getWritableDatabase();	    		
                }
                else
                {
                    cacheDB = SQLiteDatabase.openDatabase(mDBPath, null, SQLiteDatabase.OPEN_READWRITE);
                }
            }
            catch(SQLiteException e) 
            {
                    cacheDB = null;
                    Log.e(LOGTAG, e.getMessage());
            }
        }

        if (cacheDB != null)
        {
            // Search key first
            String queryString = "SELECT count(*) FROM " + TABLE_NAME;		
            try
            {
                Cursor c1 = cacheDB.rawQuery(queryString, null);
                if (c1 != null )
                {
                    c1.moveToFirst();
                    totalCachedFiles = c1.getInt(0);			

                    c1.close();
                }
            }				
            catch(SQLiteException e){
                totalCachedFiles = 0;
            }
        }
        String query = "UPDATE " + TABLE_NAME + 
        " SET " + TIMESTAMP + "=?" +
        " WHERE " + URLKEY + "=?";
        updateFileQuery = cacheDB.compileStatement(query);
        query = "INSERT INTO " + TABLE_NAME + 
        " (" + URLKEY + "," + FILENAME + "," + EXPIRATION + "," + TIMESTAMP + ")" +
        " VALUES(?, ?, ?, ?)";
        insertFileQuery = cacheDB.compileStatement(query);
    }


    synchronized public void updateFile(String url)
    {
        if (cacheDB == null || url == null) {
            return;
        }
        try
        {
            updateFileQuery.bindLong(1, System.currentTimeMillis());
            updateFileQuery.bindString(2, url);
            updateFileQuery.execute();
        }
        catch (SQLiteException e) {}
    }
        

    synchronized public void insertFile(String url, byte[] content, long expiration) 
    {
        String fileName = FileDBCacheHelper.UUID();
        long rowId = -1;
        try
        {
            insertFileQuery.bindString(1, url);
            insertFileQuery.bindString(2, fileName);
            insertFileQuery.bindLong(3, expiration);
            insertFileQuery.bindLong(4, System.currentTimeMillis());
            rowId = insertFileQuery.executeInsert();
        }
        catch (SQLiteException e) {}
        
        if (rowId == -1) {
            if(debugMode) {
                Log.d(LOGTAG, "insert image failed: " + url);
            }
        }
        else
        {
            //update file.

            File tile = new File(filePath, fileName); 
            if (!tile.exists())
            {
                try {
                    tile.createNewFile();
                    totalCachedFiles++;
                } catch (IOException e1) {						
                }
                try {
                    BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(tile));
                    try 
                    {
                        if(content != null)
                        {
                            out.write(content, 0, content.length);
                        }
                        out.close();						

                    } catch (IOException e) {

                    }
                } catch (FileNotFoundException e) {}

                if (totalCachedFiles > MAX_SIZE )
                {
                    clear(false);
                }
            }
        }
    }

    synchronized public void clear(boolean clearAll)
    {
        Cursor c = null;
        try
        {
            String sql;
            if(clearAll)
            {
                sql = "select " + FILENAME + " from " + TABLE_NAME;
            }
            else
            {
                sql = "select " + FILENAME + " from " + TABLE_NAME + 
                " order by " + TIMESTAMP + " asc limit " + DEL_NUM;
            }
            c = cacheDB.rawQuery(sql, null);
            if (c != null)
            {
                boolean hasMore = c.moveToFirst();
                Vector<String> files = new Vector<String>();
                while(hasMore)
                {                           
                    String fileToDelete = c.getString(0);
                    if (fileToDelete != null && fileToDelete.length() >  0)
                        files.add(fileToDelete);
                    hasMore = c.moveToNext();
                }

                if (files.size() > 0)
                    deleteFiles(files);
            }
        }
        finally
        {
            if (c != null)
                c.close();
        }
    }

    void deleteFiles(Vector<String>files)
    {
        Enumeration<String> iter = files.elements();
        StringBuilder whereClause = new StringBuilder();
        boolean firstElement = true;
        while(iter.hasMoreElements())
        {
            String fileName = iter.nextElement();
            File file = new File(filePath, fileName); 
            if (file.delete())
                totalCachedFiles--;

            if (!firstElement)
                whereClause.append(" OR ");
            else
                firstElement = false;
            whereClause.append(FILENAME + "='" + fileName +"'");
        }

        int deleted = cacheDB.delete(TABLE_NAME, whereClause.toString(), null);	
        if (deleted != files.size())
        {
            Log.d("Error", "should delete " + files.size() + "records instead of " + deleted );
        }
    }

    synchronized public byte[] getFile(String url) {
        byte [] data = null;
        if (cacheDB == null || url == null) {
            return null;
        }
        // Read entries from database.
        final String where = URLKEY + " = ?";
        final String[] whereArgs = { url };

        Cursor reader = null;
        try {
            reader = cacheDB.query(TABLE_NAME, null, where, whereArgs, null, null, null);
            if (reader != null) 
            {
                final int count = reader.getCount();
                if (count == 1) 
                {
                    final int columnIdFileName = reader.getColumnIndex(FILENAME);
                    final int columnIdExpiration = reader.getColumnIndex(EXPIRATION);
                    reader.moveToFirst();
                    String fileName = reader.getString(columnIdFileName);
                    long expiration = reader.getLong(columnIdExpiration);
                    if(expiration > System.currentTimeMillis())
                    {
                        File file = new File(filePath, fileName); 
                        if (file.exists())
                        {
                            try 
                            {
                                data = HttpDefaultResponse.getResponseByteArray(new FileInputStream(file));
                            } 
                            catch (FileNotFoundException e) {}
                        }
                    }
                    else
                    // Remove expired entry
                    {
                        Vector<String> toRemove = new Vector<String>();
                        toRemove.add(fileName);
                        deleteFiles(toRemove);
                    }
                }  
                else if (count > 1)
                {
                    throw new SQLiteException("More than one entry was found with url: " + url);
                }
            }
        } catch (SQLiteException e)
        {
        }
        finally {
            if (reader != null) {
                reader.close();
                reader = null;
            }
        }
        return data;
    }

    @Override
    synchronized public void onCreate(SQLiteDatabase db) {
        cacheDB = db;
        // Create table
        String sql = "create table " + TABLE_NAME + "( " + 
        BaseColumns._ID	+ " integer primary key autoincrement, " +
        URLKEY + " text not null, " +
        FILENAME + " text not null, " +
        EXPIRATION + " integer, " +
        TIMESTAMP + " integer);";

        if(debugMode) {
            Log.i(LOGTAG, "CacheDB is created.");
            Log.d(LOGTAG, "create table: " + sql);
        }

        db.execSQL(sql);

        // Create key index
        sql = "create unique index " + URLKEYINDEX + " ON " + TABLE_NAME + "(" + URLKEY + " ASC);";
        if(debugMode) {
            Log.d(LOGTAG, "create index: " + sql);
        }
        db.execSQL(sql);
    }

    @Override
    public synchronized void close() {
        if(cacheDB != null) {
            cacheDB.close();
            cacheDB = null;
            super.close();

            if(debugMode) {
                Log.i(LOGTAG, "CacheDB is closed.");
            }
        }
    }

    @Override
    synchronized public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
    }

    public synchronized static FileDBCacheHelper instance()
    {
        if(instance == null)
        {
            assert context != null : "Must call setContext() before using this singleton";
            instance = new FileDBCacheHelper(context);
        }
        return instance;
    }

    private static String UUID()
    {
        return UUID.randomUUID().toString();	
    }
}
