package com.microsoft.mobileexperiences.bing.httpthreadpool;

import java.io.IOException;
import java.io.InputStream;

public class HttpResponseBase {
	public static int SUCCESS = 0; 
	public static int ERROR = -1; 
	public static int TIMEOUT = -2; 
	public static int EXCEPTION = -3;
	
	public static int NO_ERROR_CODE = 0;

	private int statusCode; // one of the values above. 
	private int errorCode; 
	private String errorDesc;
	
	private static final int DEFAULT_BUFFER_SIZE = 16 * 1024;  
	private static final int MAX_BUFFER_SIZE = 2048 * 1024; //2M 
	
	public HttpResponseBase()
	{
		this(SUCCESS, NO_ERROR_CODE);
	}
	public HttpResponseBase(int statusCode, int errorCode)
	{
		this.statusCode = statusCode;
		this.errorCode = errorCode;
	}
	
	public HttpResponseBase(int statusCode, int errorCode, String errorDesc)
	{
		this.statusCode = statusCode;
		this.errorCode = errorCode;
		this.errorDesc = errorDesc;
	}
	
	public int getStatusCode()
	{
		return statusCode;
	}

	public int getErrorCode()
	{
		return errorCode;
	}
	
	public String getErrorDescription()
	{
		return errorDesc;
	}
	
	public void setStatusCode(int statusCode)
	{
		this.statusCode = statusCode;
	}
	
	public void setErrorCode(int errorCode)
	{
		this.errorCode = errorCode;
	}
	
	static public String getResponseString(InputStream responseStream)
	{
		byte[] buffer = getResponseByteArray(responseStream);
		return (buffer == null) ? null : new String(buffer, 0, buffer.length);
	}
	
	//limit to 2 M bytes.
	static public byte[] getResponseByteArray(InputStream responseStream)
	{		
		final int TOO_SMALL_SIZE = 100;
        int bytesread = 0;
        int actual = 0;
        int totalLength = 0;
		try {
			totalLength = responseStream.available();  //add one more byte to avoid re-allocate large trunk of data.
		} catch (IOException e1) {}
		
		if (totalLength <= TOO_SMALL_SIZE) //give a bigger buffer if the available is too small
		   	totalLength = DEFAULT_BUFFER_SIZE;		  
        
        byte[] buffer = new byte[totalLength];  
        byte[] testByte = {0};
               
        while (true) {                   
            try {
				actual = responseStream.read(buffer, bytesread, totalLength - bytesread);
			} catch (IOException e) {}
            
			if ( actual >= 0 && bytesread < MAX_BUFFER_SIZE)
            {
                bytesread += actual;
                if ( bytesread == totalLength)
                {
                	 try {
         				actual = responseStream.read(testByte, 0, 1);
         			} catch (IOException e) {}
         			
         			if (actual > 0) //has more data
         			{       
         				int available = 0 ;
         				try {
							available = responseStream.available();
						} catch (IOException e) {}
         				
						if (available > TOO_SMALL_SIZE)
         					totalLength += available;
         				else
         					totalLength += DEFAULT_BUFFER_SIZE;
	                    byte[] newBuffer = new byte[totalLength];
	                    System.arraycopy(buffer, 0, newBuffer, 0, bytesread);
	                    newBuffer[bytesread] = testByte[0];
	                    bytesread++;
	                    
	                    buffer = newBuffer; 	                  
         			}
         			else
         				break;
                }
            }
            else
                break;
        }  
        
        if (bytesread > 0)
        {   
        	if (bytesread != totalLength)
        	{
	    		byte[] newBuffer = new byte[bytesread];
	    		System.arraycopy(buffer,0, newBuffer, 0, bytesread);
	    		
	    		return newBuffer; 
        	}
        	else
        		return buffer;
        }        	
        else 
        	return null;
        
    }
}
