package com.microsoft.mobileexperiences.bing.httpthreadpool;

import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

public class HttpThreadPool{	
	
	private static final int HP_POOL_SIZE = 5;
	private static final int LP_POOL_SIZE = 6;
	public static String activeNetworkInfo;
	
    /* Get the instance of the singleton class.
    * 
    * @return <code>HttpThreadPool</code>.
    */
    public static HttpThreadPool instance() {
        return threadPoolInstance;
    }
   
    public void sendRequest(HttpRequest request, IHttpResult callback)
    {
    	request.setCallback(callback);
    	
    	if (request.getPriority() == HttpRequest.Priority.HIGH)
    		request.setFuture(highPriorityPool.submit(request));
    	else
    		request.setFuture(lowPriorityPool.submit(request));
    }
    
    public void scheduleRequest(HttpRequest request, IHttpResult callback, int delay) //delay in milliseconds
    {
    	request.setCallback(callback);
    	if (request.getPriority() == HttpRequest.Priority.HIGH)
    		request.setFuture(highPriorityPool.schedule(request, delay, TimeUnit.MILLISECONDS));
    	else
    		request.setFuture(lowPriorityPool.schedule(request, delay, TimeUnit.MILLISECONDS));
    }
    
    public void schedule(Runnable request, int delay, TimeUnit timeUnit) //delay in milliseconds
    {
    	if (request instanceof HttpRequest && ((HttpRequest)request).getPriority() == HttpRequest.Priority.HIGH)
    		((HttpRequest)request).setFuture(highPriorityPool.schedule(request, delay, timeUnit));
	    else
	    	lowPriorityPool.schedule(request, delay, timeUnit);
    	
    }
    
    
    /**
    * Constructor.
    * Make it private for singleton implementation.
    */
    private  HttpThreadPool() {      
    }    
   
    /**
    * Private instance of the singleton class
    */
    private static HttpThreadPool threadPoolInstance = new HttpThreadPool();  
    
    private ScheduledThreadPoolExecutor highPriorityPool = new ScheduledThreadPoolExecutor(HP_POOL_SIZE) ;
    private ScheduledThreadPoolExecutor lowPriorityPool = new ScheduledThreadPoolExecutor(LP_POOL_SIZE);
   
}
