package com.microsoft.mobileexperiences.bing.unittests;
import com.microsoft.mobileexperiences.bing.httpthreadpool.HttpRequest;
import com.microsoft.mobileexperiences.bing.httpthreadpool.HttpResponseBase;
import com.microsoft.mobileexperiences.bing.httpthreadpool.HttpThreadPool;
import com.microsoft.mobileexperiences.bing.httpthreadpool.IHttpResult;

import junit.framework.TestCase;


public class httpthreadpooltests  extends TestCase{
	
	class BkImageRequest extends HttpRequest
	{
		public BkImageRequest()
		{
			super( "http://appserver.m.bing.net/BackgroundImageService/TodayImageService.svc/GetTodayImage?osName=iphone&osVersion=7.0&deviceName=androidprot&orientation=480x640",
					HttpRequest.HttpMethod.HTTP_METHOD_GET);
		}		
	
		public void setHttpHeaders()
		{			
		}
	}
	
	/**
     * Basic test to request test
     */
    public void testBkImageRequest() {
        try {
        		BkImageRequest req = new BkImageRequest();
        		req.run();
        		HttpResponseBase res = req.getResponse();
        		assert(res != null);
        		assert(res.getStatusCode() == HttpResponseBase.SUCCESS);
                           
            } catch (Exception e) {
                throw new RuntimeException("Problem creating the player");
        }
    }
    
    class SimpleSearchRequest extends HttpRequest
	{    	
		public SimpleSearchRequest()
		{
			super( "http://api.m.bing.net/SearchService/Search.svc/json/", HttpRequest.HttpMethod.HTTP_METHOD_POST);
		}
		
		public String encode()
		{			
			return "{\"Context\":{\"ApplicationId\":\"E6639D72-6593-47b5-9F90-F223479563CB\",\"Culture\":\"en-us\",\"PartnerId\":\"CDA5DCA6-D955-4c28-A682-810022BC9D74\",\"RequestId\":\"e0d92355-59e3-4ad1-aa66-c0db2a54f6b1\",\"UserId\":\"a\"},\"Properties\":[],\"Queries\":[{\"Domains\":[{\"Domain\":1,{\"Range\":{\"Count\":10,\"Start\":0}}],\"Locations\": [{ \"DetectionMethod\":0,\"LocationInformation\":{\"BoundingRectangle\": {\"Northeast\":{\"Latitude\":47.0,\"Longitude\":-122.0},\"Southwest\":{\"Latitude\":47.4,\"Longitude\":-122.5}},\"Coordinate\":{\"Latitude\":47.196,\"Longitude\":-122.323},\"Name\":\"Seattle\",\"Offset\":0}}],\"Properties\":[],\"QueryString\":\"Pizza\"}],\"SafeSearch\":1}";
		}
		public void setHttpHeaders()
		{
			addHttpHeader("Content-Type", "application/json"); 
			addHttpHeader("Accept-Encoding", "gzip");      
		}
	}
    
    /**
     * Basic test to request test
     */
    public void testSimpleSearchRequest() {
        try {
        		SimpleSearchRequest req = new SimpleSearchRequest();
	        	req.run();
	        	
	        	HttpResponseBase res = req.getResponse();
        		assert(res != null);
        		assert(res.getStatusCode() == HttpResponseBase.SUCCESS);
                           
            } catch (Exception e) {
                throw new RuntimeException("Problem creating the player");
        }
    }
    
    class TestCallback implements IHttpResult
    {

		@Override
		public void onResult(HttpResponseBase response) {
			
			assert(response != null);
    		assert(response.getStatusCode() == HttpResponseBase.SUCCESS);
    		gotResponse = true;
    		count++;
		}
		public boolean gotResponse = false;
		public int count = 0;
    }
    
    
    public void testHttpThreadPoolForOneRequest() {
        try {
	        	BkImageRequest req = new BkImageRequest();
	        	
	        	TestCallback callback = new TestCallback();
	    		
	        	HttpThreadPool.instance().sendRequest(req, callback);
	        	
	        	int count = 30;
	        	while (!callback.gotResponse && count-- > 0)
	        	{
	        		Thread.sleep(1000);
	        	}
	        	
	        	assert(callback.gotResponse == true);
                           
            } catch (Exception e) {
                throw new RuntimeException("Problem creating the player");
        }
    }
    
    public void testHttpThreadPoolForMultipleRequests() {
        try {
	        	BkImageRequest req = new BkImageRequest();
	        	
	        	TestCallback callback = new TestCallback();
	    		
	        	for (int i = 0; i < 10; i++)
	        		
	        		HttpThreadPool.instance().sendRequest(req, callback);
	        	
	        	int count = 30;
	        	while (!callback.gotResponse && count-- > 0)
	        	{
	        		Thread.sleep(1000);
	        	}
	        	
	        	assert(callback.count == 10);
                           
            } catch (Exception e) {
                throw new RuntimeException("Problem creating the player");
        }
    }
    
    public void testHttpThreadPoolForMultipleRequests2() {
        try {
	        	BkImageRequest req = new BkImageRequest();
	        	SimpleSearchRequest req2 = new SimpleSearchRequest();
	        	
	        	TestCallback callback = new TestCallback();
	    		
	        	for (int i = 0; i < 5; i++){	        		
	        		HttpThreadPool.instance().sendRequest(req, callback);
	        		HttpThreadPool.instance().sendRequest(req2, callback);
	        	}
	        	
	        	int count = 30;
	        	while (!callback.gotResponse && count-- > 0)
	        	{
	        		Thread.sleep(1000);
	        	}
	        	
	        	assert(callback.count == 10);
                           
            } catch (Exception e) {
                throw new RuntimeException("Problem creating the player");
        }
    }
    
}
